"""EnergyPlusWrapper

EnergyPlus wrapper for Python

Note:

    This project is in beta stage.

Viewing documentation using IPython
-----------------------------------
To see which functions are available in `epw`, type ``epw.<TAB>`` (where
``<TAB>`` refers to the TAB key), or use ``epw.*get_version*?<ENTER>`` (where
``<ENTER>`` refers to the ENTER key) to narrow down the list.  To view the
docstring for a function, use ``epw.get_version?<ENTER>`` (to view the
docstring) and ``epw.get_version??<ENTER>`` (to view the source code).
"""

import warnings

try:
    import epw.data
except:
    warnings.warn("The epw.data submodule is not installed")

import epw.core
import epw.weather

# PEP0440 compatible formatted version, see:
# https://www.python.org/dev/peps/pep-0440/
#
# Generic release markers:
# X.Y
# X.Y.Z # For bugfix releases  
# 
# Admissible pre-release markers:
# X.YaN # Alpha release
# X.YbN # Beta release         
# X.YrcN # Release Candidate   
# X.Y # Final release
#
# Dev branch marker is: 'X.Y.dev' or 'X.Y.devN' where N is an integer.
# 'X.Y.dev0' is the canonical version of 'X.Y.dev'
#
__version__ = '1.2.dev1'

def get_version():
    return __version__

__all__ = [
            'data',
            'core'
        ]



#!/usr/bin/env python3

import epw
import matplotlib.pyplot as plt

sub_dict = {
        ("Material", "BETON 18CM", "Conductivity"): 0.5,
        ("Zone", "", "Ceiling Height"): 20.
    }

df = epw.core.sub_run(epw.data.idf_cube_path(),
                      weather_file_path=epw.data.weather_san_francisco_tmy_path(),
                      sub_dict=sub_dict)

df['DistrictHeating:Facility [J](Hourly)'].plot(figsize=(20, 8))
plt.show()


#!/usr/bin/env python3

import epw
import os
import numpy as np
import matplotlib.pyplot as plt
import tempfile

weather_obj = epw.weather.Weather()
weather_obj.read(file_path=epw.data.weather_san_francisco_tmy_path())

weather_obj.dataframe['Dry Bulb Temperature'] = np.sin(2. * np.pi / 24. * weather_obj.dataframe.index) * 10. + 12.

# WRITE THE MODIFIED WEATHER FILE ###########################################

home_path = epw.core.expand_path("~")

with tempfile.TemporaryDirectory(dir=home_path, prefix=".", suffix="_test") as temp_dir_path:
    dst_epw_path = os.path.join(temp_dir_path, "weather.epw")

    weather_obj.write(dst_epw_path)

    print("Write", dst_epw_path)

    df = epw.core.run_eplus(epw.data.idf_cube_path(),
                            weather_file_path=dst_epw_path)    # TODO

    df['DistrictHeating:Facility [J](Hourly)'].plot(figsize=(20, 8))
    plt.show()


#!/usr/bin/env python3

import epw
import matplotlib.pyplot as plt

sub_dict = {
        ("Zone", "ZONE1", "Ceiling Height"): [15., 20.],
        ("Material", "BETON 18CM", "Conductivity"): [0.5, 0.3],
        ("Material", "BETON 20CM", "Conductivity"): [0.8, 0.2]
    }

df_list = epw.core.sub_run(epw.data.idf_cube_path(),
                           weather_file_path=epw.data.weather_san_francisco_tmy_path(),
                           sub_dict=sub_dict)

print(df_list)

for df in df_list:
    df['DistrictHeating:Facility [J](Hourly)'].plot(figsize=(20, 8))

plt.show()


"""
Source of this module: https://github.com/building-energy/epw by Steven Firth and Patrick Kastner (license MIT) 
"""

import csv
import pandas as pd

class Weather:
    """
    A wrapper for EnergyPlus weather files (.epw)
    """

    def __init__(self):
        self.headers = {}
        self.dataframe = None


    def read(self, file_path):
        """
        Read the given epw file.
        """
        self.headers = self._read_metadata(file_path)
        self.dataframe = self._read_data(file_path)


    def _read_metadata(self, file_path):
        """
        Read the headers of the given epw file.
        """

        d = {}
        with open(file_path, newline='') as csvfile:
            csvreader = csv.reader(csvfile, delimiter=',', quotechar='"')
            for row in csvreader:
                if row[0].isdigit():
                    break
                else:
                    d[row[0]] = row[1:]
        return d


    def _read_data(self,fp):
        """
        Read the climate data of the given epw file.
        """

        names=['Year',
               'Month',
               'Day',
               'Hour',
               'Minute',
               'Data Source and Uncertainty Flags',
               'Dry Bulb Temperature',
               'Dew Point Temperature',
               'Relative Humidity',
               'Atmospheric Station Pressure',
               'Extraterrestrial Horizontal Radiation',
               'Extraterrestrial Direct Normal Radiation',
               'Horizontal Infrared Radiation Intensity',
               'Global Horizontal Radiation',
               'Direct Normal Radiation',
               'Diffuse Horizontal Radiation',
               'Global Horizontal Illuminance',
               'Direct Normal Illuminance',
               'Diffuse Horizontal Illuminance',
               'Zenith Luminance',
               'Wind Direction',
               'Wind Speed',
               'Total Sky Cover',
               'Opaque Sky Cover (used if Horizontal IR Intensity missing)',
               'Visibility',
               'Ceiling Height',
               'Present Weather Observation',
               'Present Weather Codes',
               'Precipitable Water',
               'Aerosol Optical Depth',
               'Snow Depth',
               'Days Since Last Snowfall',
               'Albedo',
               'Liquid Precipitation Depth',
               'Liquid Precipitation Quantity']

        first_row = self._first_row_with_climate_data(fp)
        df = pd.read_csv(fp,
                         skiprows=first_row,
                         header=None,
                         names=names)

        return df


    def _first_row_with_climate_data(self,fp):
        """Finds the first row with the climate data of an epw file
        """

        with open(fp, newline='') as csvfile:
            csvreader = csv.reader(csvfile, delimiter=',', quotechar='"')
            for i,row in enumerate(csvreader):
                if row[0].isdigit():
                    break
        return i


    def write(self, file_path):
        """
        Write data in the given epw file.
        """

        with open(file_path, 'w', newline='') as csvfile:
            csvwriter = csv.writer(csvfile,
                                   delimiter=',',
                                   quotechar='"',
                                   quoting=csv.QUOTE_MINIMAL)

            for k, v in self.headers.items():
                csvwriter.writerow([k] + v)

            for row in self.dataframe.itertuples(index=False):
                csvwriter.writerow(i for i in row)


#!/usr/bin/env python3

import epw
import matplotlib.pyplot as plt

weather_obj = epw.weather.Weather()
weather_obj.read(file_path=epw.data.weather_san_francisco_tmy_path())

df = weather_obj.dataframe
print(df.columns)

df.plot(y=['Dry Bulb Temperature', 'Dew Point Temperature'], figsize=(20, 8))
plt.show()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module contains unit tests for the powerneedsaggr module.
"""

import epw

import os
import pytest


DIR_PATH = os.path.abspath(os.path.dirname(__file__))
CUBE1_ORIG_IDF_PATH = os.path.join(DIR_PATH, "cube1.idf")
CUBE2_ORIG_IDF_PATH = os.path.join(DIR_PATH, "cube2.idf")


def test_sub_one_field_1():

    sub_dict = {
        ("Material", "BETON 18CM", "Conductivity"): 0.5
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_1.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str



def test_sub_one_field_1_int():

    sub_dict = {
        ("Material", "BETON 18CM", "Conductivity"): 1
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_1b.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str



def test_sub_one_field_1_float_without_decimal():

    sub_dict = {
        ("Material", "BETON 18CM", "Conductivity"): 1.
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_1c.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str


def test_sub_one_field_1_float_without_int():

    sub_dict = {
        ("Material", "BETON 18CM", "Conductivity"): .5
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_1.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str


def test_sub_one_field_1_list_of_len_1():

    sub_dict = {
        ("Material", "BETON 18CM", "Conductivity"): [0.5]
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_1.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str



def test_sub_one_field_2_semicolon():

    sub_dict = {
        ("InternalMass", "Inertia", "Total area exposed to zone"): 100
    }

    returned_idf_str = epw.core.sub_one_field(CUBE2_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_2.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str



def test_sub_one_field_3():

    sub_dict = {
        ("Material", "BETON 18CM", "Conductivity"): 0.5,
        ("Material", "BETON 20CM", "Conductivity"): 0.6
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_3.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str



def test_sub_one_field_4_str_value():

    sub_dict = {
        ("BuildingSurface", "PLANCHER", "Sun Exposure"): "SunExposed"
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_4.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str



def test_sub_one_field_5_colon_and_curly_bracket():

    sub_dict = {
        ("WindowMaterial:SimpleGlazingSystem", "VITRE", "U-Factor {W/m2-K}"): 1.0
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_5.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str



def test_sub_one_field_6_special_char_in_value():

    sub_dict = {
        ("ZoneInfiltration:DesignFlowRate", "Infiltrations", "Design Flow Rate Calculation Method"): "Flow*Area"
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_6.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str


def test_sub_one_field_7():

    sub_dict = {
        ("Zone", "", "Ceiling Height"): 20.
    }

    returned_idf_str = epw.core.sub_one_field(CUBE1_ORIG_IDF_PATH, sub_dict=sub_dict)

    with open(os.path.join(DIR_PATH, "expected_test_sub_one_field_7.idf")) as fd:
        expected_idf_str = fd.read()

    assert returned_idf_str == expected_idf_str


__all__ = ['expand_path']

import os
import re
import subprocess
import tempfile
import pandas as pd

EPLUS_EXEC = "energyplus"


IDF_GENERATED_FILE_NAME = "generated_input.idf"

def expand_path(path):
    """Expand `~` constructs and return an absolute path.

    On Unix operating systems (Linux, MacOSX, ...),
    `~` is a shortcut for the curent user's home directory
    (e.g. "/home/jack" for "jack").

    Example
    -------

    Lets assume we are on a Unix system,
    the current user is "jack" and the current path is "/bar".

    >>> expand_path("~/foo")
    "/home/jack/foo"

    >>> expand_path("baz")
    "/bar/baz"

    >>> expand_path("../tmp")
    "/tmp"

    Parameters
    ----------
    path : str
        The `path` to expand.

    Returns
    -------
    str
        The absolute and expanded `path`.
    """
    if path is not None:
        path = os.path.expanduser(path)  # to handle "~/..." paths
        path = os.path.abspath(path)     # to handle relative paths
        return path
    else:
        return None


def sub_run(orig_idf_path, weather_file_path, sub_dict={}, verbose=False):
    """Substitute IDF input and run eplus"""

    # CHECK sub_dict ##########################################################

    all_seq = all([type(v) in (list, tuple) and len(v) > 1 for v in sub_dict.values()])

    if all_seq:
        val_len = [len(v) for v in sub_dict.values()]
        assert min(val_len) == max(val_len)

    # RUN

    if all_seq:
        list_of_dict = [dict(zip(sub_dict, t)) for t in zip(*sub_dict.values())]
        res = [sub_one_field_and_run(orig_idf_path, weather_file_path, sub_dict2, verbose=verbose) for sub_dict2 in list_of_dict]
    else:
        res = sub_one_field_and_run(orig_idf_path, weather_file_path, sub_dict, verbose=verbose)
    
    return res


def sub_one_field(orig_idf_path, sub_dict={}, verbose=False):
    """Substitute IDF input and run eplus"""

    for k, v in sub_dict.items():
        if type(v) in (list, tuple) and len(v) == 1:
            sub_dict[k] = v[0]
    
    # CHECK sub_dict ##########################################################

    all_scalar = all([type(v) in (int, float, bool, str) for v in sub_dict.values()])
    assert all_scalar

    # READ THE ORIGINAL INPUT FILE ############################################

    idf_str = None
    orig_idf_path = expand_path(orig_idf_path)

    with open(orig_idf_path) as fd:
        idf_str = fd.read()

    # MODIFY THE INPUT FILE ###################################################

    for (obj, name, field), value in sub_dict.items():
        idf_str = re_substitute(idf_str, obj, name, field, value)

    return idf_str


def sub_one_field_and_run(orig_idf_path, weather_file_path, sub_dict={}, verbose=False):
    """Substitute IDF input and run eplus"""

    idf_str = sub_one_field(orig_idf_path, sub_dict=sub_dict, verbose=verbose)

    # WRITE THE MODIFIED INPUT FILE ###########################################

    home_path = expand_path("~")

    with tempfile.TemporaryDirectory(dir=home_path, prefix=".", suffix="_test") as temp_dir_path:
        dst_idf_path = os.path.join(temp_dir_path, IDF_GENERATED_FILE_NAME)

        if verbose:
            print("Write", dst_idf_path)

        with open(dst_idf_path, "w") as fd:
            fd.write(idf_str)

        if verbose:
            print(idf_str)

        df = run_eplus(dst_idf_path, weather_file_path=weather_file_path)

    return df


def re_substitute(s, obj, name, field, value):
    # C.f. https://www.thegeekstuff.com/2014/07/advanced-python-regex/
    #return re.sub(r'(?P<g1>' + obj + r'.*?)(?P<g2>' + name + r'.*?)(\d+.?\d*|\w+)(?P<g3>(,|;) +!- ' + field + ')',
    #              r'\g<g1>\g<g2>' + str(value) + r'\g<g3>',
    #              s,
    #              flags=re.MULTILINE|re.DOTALL)
    return re.sub(r'(?P<g1>' + obj + r'.*?)(?P<g2>' + name + r'.*?^ *)[^,;]+(?P<g3>(,|;) +!- ' + field + ')',
                  r'\g<g1>\g<g2>' + str(value) + r'\g<g3>',
                  s,
                  flags=re.MULTILINE|re.DOTALL)


def run_eplus(idf_file_path, weather_file_path, output_directory_path="."):
    """
    energyplus --help
    EnergyPlus, Version 9.4.0-998c4b761e
    PythonLinkage: Linked to Python Version: "3.6.9 (default, Jul 17 2020, 12:50:27) 
    [GCC 8.4.0]"
    Usage: energyplus [options] [input-file]
    Options:
    -a, --annual                 Force annual simulation
    -c, --convert                Output IDF->epJSON or epJSON->IDF, dependent on
                                input file type
    -D, --design-day             Force design-day-only simulation
    -d, --output-directory ARG   Output directory path (default: current
                                directory)
    -h, --help                   Display help information
    -i, --idd ARG                Input data dictionary path (default: Energy+.idd
                                in executable directory)
    -m, --epmacro                Run EPMacro prior to simulation
    -p, --output-prefix ARG      Prefix for output file names (default: eplus)
    -r, --readvars               Run ReadVarsESO after simulation
    -s, --output-suffix ARG      Suffix style for output file names (default: L)
                                    L: Legacy (e.g., eplustbl.csv)
                                    C: Capital (e.g., eplusTable.csv)
                                    D: Dash (e.g., eplus-table.csv)
    -v, --version                Display version information
    -w, --weather ARG            Weather file path (default: in.epw in current
                                directory)
    -x, --expandobjects          Run ExpandObjects prior to simulation
    --convert-only                 Only convert IDF->epJSON or epJSON->IDF,
                                dependent on input file type. No simulation
    Example: energyplus -w weather.epw -r input.idf

    """

    idf_file_path = expand_path(idf_file_path)
    weather_file_path = expand_path(weather_file_path)
    output_directory_path = expand_path(output_directory_path)

    cmd = [
            EPLUS_EXEC,
            "-w", weather_file_path,
            "-d", output_directory_path,
            "-r",
            idf_file_path
          ]

    print(" ".join(cmd))

    subprocess.call(cmd)

    df = pd.read_csv(os.path.join(output_directory_path, "eplusout.csv"))

    return df